@@ -10,13 +10,11 @@ import itertools
 import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
+from scipy.spatial import Delaunay
+from collections import OrderedDict
 
 import pynibs
 
-from scipy.spatial import Delaunay
-
-from collections import OrderedDict
-
 
 def get_coil_dipole_pos(coil_fn, matsimnibs):
     """
@@ -31,8 +29,8 @@ def get_coil_dipole_pos(coil_fn, matsimnibs):
 
     Returns
     -------
-    dipoles_pos : nparray [N x 3]
-        Cartesian coordinates (x, y, z) of coil magnetic dipoles
+    dipoles_pos : np.ndarray
+    (N, 3) Cartesian coordinates (x, y, z) of coil magnetic dipoles
 
     """
 
@@ -52,15 +50,15 @@ def check_coil_position(points, hull):
 
     Parameters
     ----------
-    points : np.ndarray of float [N_points x 3]
-        Coordinates (x,y,z) of magnetic dipoles
-    hull : Delaunay object or np.ndarray of float [N_surface_points x 3]
-        Head surface data
+    points : np.ndarray of float
+        (N_points, 3) Coordinates (x,y,z) of magnetic dipoles
+    hull : Delaunay object or np.ndarray of float
+        (N_surface_points, 3) Head surface data
 
     Returns
     -------
     valid : bool
-        Validity of coil position
+        Validity of coil position:
         TRUE: valid
         FALSE: unvalid
     """
@@ -88,10 +86,10 @@ def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
 
     Parameters
     ----------
-    LOC_mean : np.ndarray of float [3]
-        Mean location of TMS coil
-    ORI_mean : np.ndarray of float [3 x 3]
-        Mean orientations of TMS coil
+    LOC_mean : np.ndarray of float
+        (3), Mean location of TMS coil
+    ORI_mean : np.ndarray of float
+        (3 x 3) Mean orientations of TMS coil
 
         .. math::
             \\begin{bmatrix}
@@ -100,17 +98,17 @@ def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
             | & | & | \\\\
             \\end{bmatrix}
 
-    LOC_var : nparray of float [3]
-        Location variation in normalized space (dx', dy', dz'), i.e. zero mean and projected on principal axes
-    ORI_var : nparray of float [3]
-        Orientation variation expressed in Euler angles [alpha, beta, gamma] in deg
-    V : nparray of float [3x3]
-        V-matrix containing the eigenvectors from  _,_,V = numpy.linalg.svd
+    LOC_var : np.ndarray of float
+        (3) Location variation in normalized space (dx', dy', dz'), i.e. zero mean and projected on principal axes
+    ORI_var : np.ndarray of float
+        (3) Orientation variation expressed in Euler angles [alpha, beta, gamma] in deg
+    V : np.ndarray of float
+        (3x3) V-matrix containing the eigenvectors from  _,_,V = numpy.linalg.svd
 
     Returns
     -------
-    mat : nparray of float [4 x 4]
-        Transformation matrix containing 3 axis and 1 location vector:
+    mat : np.ndarray of float
+    (4, 4) Transformation matrix containing 3 axis and 1 location vector:
 
         .. math::
             \\begin{bmatrix}
@@ -309,8 +307,8 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
             elif orientation == 'euler':
                 orientations[i][j, :] = pynibs.rotation_matrix_to_euler_angles(op_zeromean[i][:, :, j]) * 180.0 / np.pi
                 ori_names = ['Psi', 'Theta', 'Phi']
-                xlabel = ['$\Psi$', '$\Theta$', '$\Phi$']
-                ylabel = ['$p(\Psi)$', '$p(\Theta)$', '$p(\Phi)$']
+                xlabel = [r'$\Psi$', r'$\Theta$', r'$\Phi$']
+                ylabel = [r'$p(\Psi)$', r'$p(\Theta)$', r'$p(\Phi)$']
 
             else:
                 ValueError('Please set orientation parameter to either ''quaternions'' or ''euler''')
@@ -385,7 +383,7 @@ def test_coil_position_gpc(parameters):
             node_number_list = np.array(f['mesh/elm/node_number_list'])
             elm_type = np.array(f['mesh/elm/elm_type'])
             regions = np.array(f['mesh/elm/tag1'])
-            triangles_regions = regions[elm_type == 2, ] - 1000
+            triangles_regions = regions[elm_type == 2,] - 1000
             triangles = node_number_list[elm_type == 2, 0:3]
 
         triangles = triangles[triangles_regions == 5]
@@ -481,9 +479,9 @@ def get_invalid_coil_parameters(param_dict, coil_position_mean, svd_v, del_obj,
     param_dict: dict
         Dictionary containing dictionary with 'limits' and 'pdfshape'
         keys: 'x', 'y', 'z', 'psi', 'theta', 'phi'
-    coil_position_mean: nparray [3 x 4]
+    coil_position_mean: np.ndarray [3 x 4]
         Mean coil positions and orientations
-    svd_v: nparray [3 x 3]
+    svd_v: np.ndarray [3 x 3]
         SVD matrix V
     del_obj: Delaunay object
         Skin surface (created with scipy.spatial.Delaunay)
@@ -1189,8 +1187,8 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
                 else:
                     # Euclidean distance
                     distances[from_counter][to_counter] = (int(
-                            math.hypot((from_node[0] - to_node[0]),
-                                       (from_node[1] - to_node[1]))))
+                        math.hypot((from_node[0] - to_node[0]),
+                                   (from_node[1] - to_node[1]))))
         return distances
 
     def get_routes(s, r, m):
