@@ -1,16 +1,14 @@
-import platform
+import os
 import shutil
+import subprocess
+
+import h5py
+import meshio
+import nibabel
+import platform
 import numpy as np
 import nibabel as nib
-import meshio
-import os
-from .mesh import map_data_to_surface, data_nodes2elements, data_elements2nodes
-from .util import *
-from .hdf5_io import write_geo_hdf5_surf
-from .hdf5_io import write_data_hdf5_surf
-from .roi import make_GM_WM_surface
-from .subject import *
-import nibabel
+import pynibs
 
 
 def read_curv_data(fname_curv, fname_inf, raw=False):
@@ -41,7 +39,7 @@ def read_curv_data(fname_curv, fname_inf, raw=False):
     points_inf, con_inf = nibabel.freesurfer.read_geometry(fname_inf)
 
     # interpolate point data to element centers
-    curv = data_nodes2elements(curv_points, con_inf)
+    curv = pynibs.data_nodes2elements(curv_points, con_inf)
 
     # normalize curvature data (optional)
     if not raw:
@@ -100,7 +98,7 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
     for i, subject in enumerate(subjects):
         if not os.path.exists(os.path.join(subject_dir, subject_names[i])):
             print(('Generating symlink of subject {} in {}'.format(subject_names[i], subject_dir)))
-            bash('ln -s ' + subject + ' ' + subject_dir)
+            pynibs.bash('ln -s ' + subject + ' ' + subject_dir)
 
     # if not subject_dir:
     #     subject_dir = "/data/pt_01756/tmp/fs_ownav/"
@@ -220,7 +218,7 @@ def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'
         raise OSError('Average template can only be generated with linux because of FREESURFER requirements!')
 
     # check if FREESURFER is installed
-    output = bash('which FREESURFER')
+    output = pynibs.bash('which FREESURFER')
     if not output:
         raise Exception('FREESURFER not installed! Please visit https://freesurfer.net/ for more information')
 
@@ -235,7 +233,7 @@ def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'
     for i, subject in enumerate(subjects):
         if not os.path.exists(os.path.join(subject_dir, subject_names[i])):
             print(('Generating symlink of subject {} in {}'.format(subject_names[i], subject_dir)))
-            bash('ln -s ' + subject + ' ' + subject_dir)
+            pynibs.bash('ln -s ' + subject + ' ' + subject_dir)
 
     # make average template
     if os.path.exists(f'{subject_dir}{os.sep}avg_template'):
@@ -305,8 +303,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
         - fn_out_hdf5_data.hdf5: geometry file containing the data
     """
 
-    subject = load_subject(fname=fn_subject_obj)
-    average = load_subject(fname=fn_average_obj)
+    subject = pynibs.load_subject(fname=fn_subject_obj)
+    average = pynibs.load_subject(fname=fn_average_obj)
 
     mesh_folder_avg = average.mesh["0"]["mesh_folder"]
     mesh_folder_sub = subject.mesh[mesh_idx]["mesh_folder"]
@@ -339,11 +337,11 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
 
     if verbose:
         print('> Generating GM/WM surface on average template')
-    points_avg, con_avg = make_GM_WM_surface(gm_surf_fname=fn_avg_gm,
-                                             wm_surf_fname=fn_avg_wm,
-                                             mesh_folder=mesh_folder_avg,
-                                             delta=subject.roi[mesh_idx][roi_idx]['delta'],
-                                             layer=1)
+    points_avg, con_avg = pynibs.make_GM_WM_surface(gm_surf_fname=fn_avg_gm,
+                                                    wm_surf_fname=fn_avg_wm,
+                                                    mesh_folder=mesh_folder_avg,
+                                                    delta=subject.roi[mesh_idx][roi_idx]['delta'],
+                                                    layer=1)
     files_not_exist = not np.array([os.path.exists(fn_data_sub_nodes_mapped_avg_curv[i]) for
                                     i in range(len(data_label))]).all()
 
@@ -366,8 +364,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
         if subject_data_in_center:
             if verbose:
                 print('> Transforming subject data from element centers to nodes')
-            # data_sub_nodes = [data_elements2nodes(data_sub[i], con_ROI) for i in range(len(data_label))]
-            data_sub_nodes = data_elements2nodes(data_sub, con_roi)
+            # data_sub_nodes = [pynibs.data_elements2nodes(data_sub[i], con_ROI) for i in range(len(data_label))]
+            data_sub_nodes = pynibs.data_elements2nodes(data_sub, con_roi)
 
         else:
             data_sub_nodes = data_sub
@@ -379,15 +377,15 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
         if subject.roi[mesh_idx][roi_idx]['fn_mask']:
             if verbose:
                 print('> Mapping point data of ROI to whole brain surface')
-            data_sub_nodes_mapped = map_data_to_surface(datasets=data_sub_nodes,
-                                                        points_datasets=[points_roi] * len(data_sub_nodes),
-                                                        con_datasets=[con_roi] * len(data_sub_nodes),
-                                                        fname_fsl_gm=os.path.join(mesh_folder_sub, fn_sub_gm),
-                                                        fname_fsl_wm=os.path.join(mesh_folder_sub, fn_sub_wm),
-                                                        delta=subject.roi[mesh_idx][roi_idx]['delta'],
-                                                        input_data_in_center=False,
-                                                        return_data_in_center=False,
-                                                        data_substitute=data_substitute)
+            data_sub_nodes_mapped = pynibs.map_data_to_surface(datasets=data_sub_nodes,
+                                                               points_datasets=[points_roi] * len(data_sub_nodes),
+                                                               con_datasets=[con_roi] * len(data_sub_nodes),
+                                                               fname_fsl_gm=os.path.join(mesh_folder_sub, fn_sub_gm),
+                                                               fname_fsl_wm=os.path.join(mesh_folder_sub, fn_sub_wm),
+                                                               delta=subject.roi[mesh_idx][roi_idx]['delta'],
+                                                               input_data_in_center=False,
+                                                               return_data_in_center=False,
+                                                               data_substitute=data_substitute)
         else:
             data_sub_nodes_mapped = data_sub_nodes
 
@@ -448,27 +446,28 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
     if verbose:
         print('> Transforming mapped nodal data to element centers')
     for i in range(len(data_label)):
-        data_avg_centers_mapped[i] = data_nodes2elements(data_avg_nodes_mapped[i], con_avg)[:, np.newaxis]
+        data_avg_centers_mapped[i] = pynibs.data_nodes2elements(data_avg_nodes_mapped[i], con_avg)[:, np.newaxis]
 
     # create .hdf5 geometry file
     if verbose:
         print('> Creating .hdf5 geometry file of mapped data')
-    write_geo_hdf5_surf(out_fn=fn_out_hdf5_geo, points=points_avg, con=con_avg, replace=True, hdf5_path='/mesh')
+    pynibs.write_geo_hdf5_surf(out_fn=fn_out_hdf5_geo, points=points_avg, con=con_avg, replace=True, hdf5_path='/mesh')
 
     # create .hdf5 data file
     if verbose:
         print('> Creating .hdf5 data files')
-    write_data_hdf5_surf(data=data_avg_centers_mapped,
-                         data_names=data_label,
-                         data_hdf_fn_out=fn_out_hdf5_data,
-                         geo_hdf_fn=fn_out_hdf5_geo,
-                         replace=True)
+    pynibs.write_data_hdf5_surf(data=data_avg_centers_mapped,
+                                data_names=data_label,
+                                data_hdf_fn_out=fn_out_hdf5_data,
+                                geo_hdf_fn=fn_out_hdf5_geo,
+                                replace=True)
 
     if verbose:
         print('> Done!\n')
 
 
-def freesurfer2vtk(in_fns, out_folder, hem='lh', surf='pial', prefix=None, fs_subject='fsaverage', fs_subjects_dir=None):
+def freesurfer2vtk(in_fns, out_folder, hem='lh', surf='pial', prefix=None, fs_subject='fsaverage',
+                   fs_subjects_dir=None):
     f"""
     Transform multiple freesurfer .mgh files into one .vtk file.
     This can be read with Paraview and others.
