@@ -1,8 +1,7 @@
 import numpy as np
-from pynibs import load_muaps
-from .exp.Mep import sigmoid
 from scipy.special import logsumexp
 from scipy.interpolate import LinearNDInterpolator
+import pynibs
 
 
 def calc_e_sensitivity(theta, gradient):
@@ -150,7 +149,7 @@ def DI_wave(t, intensity, t0=5, dt=1.4, width=0.25):
     for i, w in enumerate(waves):
         y_ = np.exp(-(t - t0 - i * dt) ** 2 / (2 * width ** 2))
         y_ = y_ / np.max(y_)
-        y_ = y_ * sigmoid(intensity, amp=amp[w], r=r[w], x0=x0[w])
+        y_ = y_ * pynibs.sigmoid(intensity, amp=amp[w], r=r[w], x0=x0[w])
         y = y + y_
 
     return y
@@ -254,7 +253,7 @@ class Model(object):
         self.fn_muaps = fn_muaps
         self.muaps = []
         self.t_muap = []
-        self.muaps, t_muap = load_muaps(self.fn_muaps)
+        self.muaps, t_muap = pynibs.load_muaps(self.fn_muaps)
 
         # Workhorse Arrays
         self.alpha_motor_neurons = []
