@@ -1,23 +1,14 @@
 import os
 import h5py
-from .regression import *
 import scipy
 import pynibs
 import numpy as np
-import pandas as pd
-from .exp.Mep import linear
-from .exp.Mep import sigmoid
-from .hdf5_io import write_geo_hdf5
-from .hdf5_io import write_temporal_xdmf
-from .hdf5_io import load_mesh_hdf5
 import multiprocessing.pool
 import matplotlib.pyplot as plt
 from _functools import partial
 from sklearn.neighbors import KernelDensity
 
 
-# Workhorses for multiprocessing
-###################################################################
 def workhorse_mc(idx_list, array, ele_idx_1, mode="cols"):
     """
     Determines mutual coherence for given zap indices in idx_list.
@@ -56,7 +47,15 @@ def workhorse_mc(idx_list, array, ele_idx_1, mode="cols"):
     return res
 
 
+# TODO Benjamin
 def workhorse_corr(idx_list, array, ele_idx_1):
+    """
+
+    :param idx_list:
+    :param array:
+    :param ele_idx_1:
+    :return:
+    """
     array = array[:, ele_idx_1]
 
     if array.ndim == 1:
@@ -157,13 +156,13 @@ def workhorse_variability(idx_list, array, ele_idx_1):
     res = []
 
     for ind in idx_list:
-        D = np.zeros((array.shape[1], array.shape[1]))
+        distances = np.zeros((array.shape[1], array.shape[1]))
         d = 0
 
         for col_idx in range(0, array.shape[1]-1):
             for row_idx in range(col_idx+1, array.shape[1]):
-                D[row_idx, col_idx] = np.linalg.norm(array[ind, row_idx] - array[ind, col_idx])
-                d += D[row_idx, col_idx]
+                distances[row_idx, col_idx] = np.linalg.norm(array[ind, row_idx] - array[ind, col_idx])
+                d += distances[row_idx, col_idx]
 
         res.append(1/d)
 
@@ -534,7 +533,8 @@ def workhorse_fim_svd(idx_list, array, ele_idx_1, ele_idx_2, e_opt, c=None):
 
     for j, ind in enumerate(idx_list):
         # fim
-        res_fim[j] = np.linalg.norm((e_opt - array_fim[ind[-1], :]) * c[ele_idx_1])
+        intensity = np.mean(array_fim[ind[-1], :] / e_opt)
+        res_fim[j] = np.linalg.norm((e_opt - intensity * array_fim[ind[-1], :]) * c[ele_idx_1])
 
         # svd
         u, s, vh = scipy.linalg.svd(array_svd[ind, :])
@@ -604,7 +604,7 @@ def workhorse_fim_mc(idx_list, array, ele_idx_1, ele_idx_2, e_opt, c=None, mode=
 
 
 def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_idx_2=None, fn_out_hdf5=None, n_cpu=4,
-                               zap_idx_opt=None, fun=None, p=None, c=None, weights=[0.5, 0.5],
+                               zap_idx_opt=None, fun=None, p=None, c=None, weights=None,
                                overwrite=True, verbose=True, fn_coilpos_hdf5=None, start_zap_idx=0):
     """
     Determine set of optimal coil positions for TMS regression analysis.
@@ -669,6 +669,8 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
         Output file containing the zap index lists
     """
 
+    if weights is None:
+        weights = [0.5, 0.5]
     if zap_idx_opt is not None and fn_out_hdf5 is not None and (os.path.exists(fn_out_hdf5) and not overwrite):
         raise ValueError("zap_idx_opt and fn_out_hdf5 given... please choose whether to load optimal zap indices from "
                          "file or given explicitly as list")
@@ -809,7 +811,8 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
             elif weights[1] == 0:
                 workhorse_partial = partial(workhorse_dist, array=e, ele_idx_1=ele_idx_1)
             else:
-                workhorse_partial = partial(workhorse_dist_mc, array=e, ele_idx_1=ele_idx_1, ele_idx_2=ele_idx_2, mode="cols")
+                workhorse_partial = partial(workhorse_dist_mc, array=e, ele_idx_1=ele_idx_1, ele_idx_2=ele_idx_2,
+                                            mode="cols")
 
         elif criterion == "dist_mc_rows":
             if weights[0] == 0:
@@ -817,7 +820,8 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
             elif weights[1] == 0:
                 workhorse_partial = partial(workhorse_dist, array=e, ele_idx_1=ele_idx_1)
             else:
-                workhorse_partial = partial(workhorse_dist_mc, array=e, ele_idx_1=ele_idx_1, ele_idx_2=ele_idx_2, mode="rows")
+                workhorse_partial = partial(workhorse_dist_mc, array=e, ele_idx_1=ele_idx_1, ele_idx_2=ele_idx_2,
+                                            mode="rows")
 
         elif criterion == "dist_svd":
             if weights[0] == 0:
@@ -877,7 +881,7 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
                 else:
                     res_all = np.vstack((res_all, np.vstack(res[j]).transpose()))
 
-            # filter nans
+            # filter nans # TODO: not to 1e6 -> max of each opt criterion (column)
             res_all[np.isnan(res_all)] = 1e6
             res_all[res_all == 0] = 1e-6
 
@@ -887,7 +891,7 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
             # weight optimization criteria
             res_all = res_all * weights
 
-            # find best solution with lowest sum
+            # find the best solution with the lowest sum
             res_all_sum = np.sum(res_all, axis=1)
             idx_best = np.argmin(res_all_sum)
             crit[i] = res_all_sum[idx_best]
@@ -985,8 +989,8 @@ def get_optimal_coil_positions(e_matrix, criterion, n_stim, ele_idx_1=None, ele_
 
 def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn_stimsites_hdf5, e_matrix, mep,
                         mesh_idx, roi_idx, n_zaps_init=3, criterion_init="mc_rows", criterion="coverage", n_cpu=4,
-                        threshold=0.8, weights=[0.5, 0.5], eps0=0.01, eps0_dist=1, exponent=5, perc=99,
-                        n_refit=0, fun=sigmoid, verbose=True):
+                        threshold=0.8, weights=None, eps0=0.01, eps0_dist=1, exponent=5, perc=99,
+                        n_refit=0, fun=pynibs.sigmoid, verbose=True):
     """
     Performs virtual online optimization to determine the congruence factor. After an initial set of coil positions,
     the algorithm iteratively optimizes the next coil position based on the virtually measured MEP data.
@@ -1048,6 +1052,8 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
         Results output file containing the coil positions and the congruence factor maps for every iteration
 
     """
+    if weights is None:
+        weights = [0.5, 0.5]
     print("Starting online congruence factor optimization:")
     print("===============================================")
     print(f" > fn_subject_hdf5:          {fn_subject_hdf5}")
@@ -1091,12 +1097,12 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
     with h5py.File(fn_roi_ss_indices_hdf5, "r") as f:
         ele_idx_ss = f["roi_indices"][:]
 
-    # load e-fields from reg.hdf5
+    # e-fields
     if verbose:
         print(f"Loading electric field from regression.hdf5")
     n_ele = e_matrix.shape[1]
 
-    # loading coil positions
+    # loading coil positions and create matsimnibs [4x4] matrices
     with h5py.File(fn_stimsites_hdf5, "r") as f:
         centers_all = f["centers"][:]
         m0_all = f["m0"][:]
@@ -1129,46 +1135,54 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
                                                     overwrite=False,
                                                     verbose=True)
 
-    # determine initial c-factor map
+    # determine initial c-factor map for all N (not existing in real life)
     if verbose:
         print(f"Determine reference c-factor map (N)")
 
-    c_ref_N = regression.nl_hdf5(elm_idx_list=np.arange(n_ele),
-                                 e_matrix=e_matrix,
-                                 mep=mep,
-                                 zap_idx=None,
-                                 fun=fun,
-                                 n_refit=n_refit,
-                                 n_cpu=n_cpu,
-                                 con=con,
-                                 return_fits=False)
+    c_ref_n = pynibs.regress_data(elm_idx_list=np.arange(n_ele),
+                                  e_matrix=e_matrix,
+                                  mep=mep,
+                                  zap_idx=None,
+                                  fun=fun,
+                                  n_refit=n_refit,
+                                  n_cpu=n_cpu,
+                                  con=con,
+                                  return_fits=False,
+                                  refit_discontinuities=True)
 
-    ref_N = c_ref_N.flatten()**exponent
-    ref_N = ref_N / np.percentile(ref_N, perc)
+    ref_n = c_ref_n.flatten()**exponent
+    ref_n = ref_n / np.percentile(ref_n, perc)
 
-    c_max_idx_N = np.argmax(c_ref_N)
+    c_max_idx_N = np.argmax(c_ref_n)
 
-    # determine initial c-factor map
+    ##########################################################################
+    #
+    # Robot measures initial offline optimal coil positions and collects MEPs
+    #
+    ##########################################################################
+
+    # determine initial c-factor map (after robot measured first offline optimal coil positions)
     if verbose:
         print(f"Determine initial c-factor map")
 
-    c_init, p = regression.nl_hdf5(elm_idx_list=np.arange(n_ele),
-                                   e_matrix=e_matrix,
-                                   mep=mep,
-                                   zap_idx=zap_idx_opt,
-                                   fun=fun,
-                                   n_refit=n_refit,
-                                   n_cpu=n_cpu,
-                                   con=con,
-                                   return_fits=True)
+    c_init, p = pynibs.regress_data(elm_idx_list=np.arange(n_ele),
+                                    e_matrix=e_matrix,
+                                    mep=mep,
+                                    zap_idx=zap_idx_opt,
+                                    fun=fun,
+                                    n_refit=n_refit,
+                                    n_cpu=n_cpu,
+                                    con=con,
+                                    return_fits=True,
+                                    refit_discontinuities=True)
     ref = c_init.flatten()**exponent
     ref = ref / np.percentile(ref, perc)
 
     eps = [eps0 + 1 for _ in range(len(zap_idx_opt))]
-    eps_N = [eps0 + 1 for _ in range(len(zap_idx_opt))]
+    eps_n = [eps0 + 1 for _ in range(len(zap_idx_opt))]
 
     gdist = [eps0_dist + 1 for _ in range(len(zap_idx_opt))]
-    gdist_N = [eps0_dist + 1 for _ in range(len(zap_idx_opt))]
+    gdist_n = [eps0_dist + 1 for _ in range(len(zap_idx_opt))]
 
     for i in range(len(zap_idx_opt)):
         zap_idx[str(i)] = zap_idx_opt[:(i+1)]
@@ -1178,6 +1192,7 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
 
     n_zaps = [i for i in range(1, n_zaps_init+1)]
 
+    # Start online optimization loop
     while not ((np.array(eps[-3:]) < eps0).all() and (np.array(gdist[-3:]) < eps0_dist).all()):
 
         if "fim" in criterion or "dist_" in criterion:
@@ -1213,53 +1228,67 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
         key = str(len(zap_idx_opt)-1)
         zap_idx[key] = zap_idx_opt
 
+        ##########################################################################
+        #
+        # Robot measures next optimal coil position
+        #
+        ##########################################################################
+
         # determine updated c-factor map
         if verbose:
             print(f"Determine c-factor map for {len(zap_idx_opt)} zaps")
 
-        c[key], p = regression.nl_hdf5(elm_idx_list=np.arange(n_ele),
-                                       e_matrix=e_matrix,
-                                       mep=mep,
-                                       zap_idx=zap_idx_opt,
-                                       fun=fun,
-                                       n_refit=n_refit,
-                                       n_cpu=n_cpu,
-                                       con=con,
-                                       return_fits=True)
+        c[key], p = pynibs.regress_data(elm_idx_list=np.arange(n_ele),
+                                        e_matrix=e_matrix,
+                                        mep=mep,
+                                        zap_idx=zap_idx_opt,
+                                        fun=fun,
+                                        n_refit=n_refit,
+                                        n_cpu=n_cpu,
+                                        con=con,
+                                        return_fits=True,
+                                        refit_discontinuities=True)
         arr = c[key].flatten()**exponent
         arr = arr / np.percentile(arr, perc)
 
+        ##########################################################################
+        #
+        # Plot updated c-factor map
+        #
+        ##########################################################################
+
         # determine NRMSD w.r.t. previous solution
         eps.append(pynibs.nrmsd(arr, ref))
 
         if verbose:
             print(f"NRMSD to previous solution: {eps[-1]}")
 
-        # determine NRMSD w.r.t. global solution
-        eps_N.append(pynibs.nrmsd(arr, ref_N))
+        # determine NRMSD w.r.t. global solution (not existing in real life)
+        eps_n.append(pynibs.nrmsd(arr, ref_n))
 
         if verbose:
-            print(f"NRMSD to global solution: {eps_N[-1]}")
+            print(f"NRMSD to global solution: {eps_n[-1]}")
 
         # determine geodesic distance w.r.t. previous solution
-        nodes_dist, tris_dist = pynibs.geodesic_dist(nodes=points, tris=con, source=np.argmax(c[key]), source_is_node=False)
+        nodes_dist, tris_dist = pynibs.geodesic_dist(nodes=points, tris=con, source=np.argmax(c[key]),
+                                                     source_is_node=False)
         gdist.append(tris_dist[np.argmax(c[str(len(zap_idx_opt)-2)])])
 
         if verbose:
             print(f"GDIST to previous solution: {gdist[-1]:.3f} mm")
 
-        # determine geodesic distance w.r.t. global solution
-        gdist_N.append(tris_dist[c_max_idx_N])
+        # determine geodesic distance w.r.t. global solution (not existing in real life)
+        gdist_n.append(tris_dist[c_max_idx_N])
 
         if verbose:
-            print(f"GDIST to global solution: {gdist_N[-1]:.3f} mm")
+            print(f"GDIST to global solution: {gdist_n[-1]:.3f} mm")
 
         # set current solution as ref
         ref = arr
 
-    eps_N = np.array(eps_N)
+    eps_n = np.array(eps_n)
     eps = np.array(eps)
-    gdist_N = np.array(gdist_N)
+    gdist_n = np.array(gdist_n)
     gdist = np.array(gdist)
     n_zaps = np.array(n_zaps)
 
@@ -1267,12 +1296,12 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
         print(f"Saving results to {fn_out_hdf5}")
 
     with h5py.File(fn_out_hdf5, "w") as f:
-        f.create_dataset("nrmsd", data=eps_N)
+        f.create_dataset("nrmsd", data=eps_n)
         f.create_dataset("nrmsd_n_1", data=eps)
-        f.create_dataset("gdist", data=gdist_N)
+        f.create_dataset("gdist", data=gdist_n)
         f.create_dataset("gdist_n_1", data=gdist)
         f.create_dataset("n_zaps", data=n_zaps)
-        f.create_dataset(f"c_ref", data=c_ref_N)
+        f.create_dataset(f"c_ref", data=c_ref_n)
 
         for key in zap_idx:
             f.create_dataset(f"zap_index_lists/{key}", data=zap_idx[key])
@@ -1330,11 +1359,11 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
 
     sort_idx = np.argsort(n_zaps)
     n_zaps = n_zaps[sort_idx]
-    eps_N = eps_N[sort_idx]
+    eps_n = eps_n[sort_idx]
     eps = eps[sort_idx]
 
     # nrmsd (n vs N) error
-    plt.plot(n_zaps[n_zaps_init:], eps_N[n_zaps_init:], color="r")
+    plt.plot(n_zaps[n_zaps_init:], eps_n[n_zaps_init:], color="r")
 
     # nrmsd (n vs n-1) error
     plt.plot(n_zaps[n_zaps_init:], eps[n_zaps_init:], color="b")
@@ -1356,7 +1385,7 @@ def online_optimization(fn_subject_hdf5, fn_roi_ss_indices_hdf5, fn_out_hdf5, fn
     plt.close()
 
     # gdist (n vs N) error
-    plt.plot(n_zaps[n_zaps_init:], gdist_N[n_zaps_init:], color="r")
+    plt.plot(n_zaps[n_zaps_init:], gdist_n[n_zaps_init:], color="r")
 
     # gdist (n vs n-1) error
     plt.plot(n_zaps[n_zaps_init:], gdist[n_zaps_init:], color="b")
