@@ -1,14 +1,11 @@
 import os
 import time
 import warnings
-
 import h5py
 import numpy as np
 import yaml
 import inspect
-
 from lmfit import Model
-
 from sklearn.neighbors import KernelDensity
 import pynibs
 
@@ -109,10 +106,10 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
 
                 for idx in range(n_conditions):
                     # lower boundary idx of e_x_cond in e_x_global
-                    start_ind[idx] = pynibs.find_nearest(e_x_global, e_min[idx])
+                    start_ind[idx] = pynibs.mesh.utils.find_nearest(e_x_global, e_min[idx])
 
                     # upper boundary idx of e_x_cond in e_x_global
-                    stop_ind[idx] = pynibs.find_nearest(e_x_global, e_max[idx])
+                    stop_ind[idx] = pynibs.mesh.utils.find_nearest(e_x_global, e_max[idx])
 
                 # get tau distances for all conditions vs reference condition
                 # distances for ref,i == i,ref. i,i == 0. So only compute upper triangle of matrix
@@ -159,8 +156,9 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
                                               intensities_max[idx]
 
                         # start index of initially shifted and stretched mep curve
-                        start_idx_mep_initial_shift = pynibs.find_nearest(e_x_global,
-                                                                          e_mep_initial_shift * intensities_min[idx])
+                        start_idx_mep_initial_shift = pynibs.mesh.utils.find_nearest(e_x_global,
+                                                                                     e_mep_initial_shift *
+                                                                                     intensities_min[idx])
 
                         mep_shift = mep[idx].eval(intens_mep, mep_params_cond[idx])
 
@@ -589,10 +587,10 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
 
                 for idx in range(n_conditions):
                     # lower boundary idx of e_x_cond in e_x_global
-                    start_ind[idx] = pynibs.find_nearest(e_x_global, e_min[idx])
+                    start_ind[idx] = pynibs.mesh.utils.find_nearest(e_x_global, e_min[idx])
 
                     # upper boundary idx of e_x_cond in e_x_global
-                    stop_ind[idx] = pynibs.find_nearest(e_x_global, e_max[idx])
+                    stop_ind[idx] = pynibs.mesh.utils.find_nearest(e_x_global, e_max[idx])
 
                 # get tau distances for all conditions vs reference condition
                 # distances for ref,i == i,ref. i,i == 0. So only compute upper triangle of matrix
@@ -639,8 +637,8 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
                                               intensities_max[idx]
 
                         # start index of initially shifted and stretched mep curve
-                        start_idx_mep_initial_shift = pynibs.find_nearest(e_x_global,
-                                                                          e_mep_initial_shift * intensities_min[idx])
+                        start_idx_mep_initial_shift = pynibs.mesh.utils.find_nearest(e_x_global,
+                                                                                     e_mep_initial_shift * intensities_min[idx])
 
                         mep_shift = mep[idx].eval(intens_mep, mep_params_cond[idx])
 
@@ -1617,15 +1615,15 @@ def extract_condition_combination(fn_config_cfg, fn_results_hdf5, conds, fn_out_
 
     # map data to whole brain surface
     print(" > Mapping c-factor map to brain surface...")
-    c_mapped = pynibs.map_data_to_surface(datasets=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
-                                          points_datasets=[roi[roi_idx].node_coord_mid] * n_qoi,
-                                          con_datasets=[roi[roi_idx].node_number_list] * n_qoi,
-                                          fname_fsl_gm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
-                                          fname_fsl_wm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
-                                          delta=subject.roi[mesh_idx][roi_idx]['delta'],
-                                          input_data_in_center=True,
-                                          return_data_in_center=True,
-                                          data_substitute=-1)
+    c_mapped = pynibs.mesh.transformations.map_data_to_surface(datasets=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
+                                                               points_datasets=[roi[roi_idx].node_coord_mid] * n_qoi,
+                                                               con_datasets=[roi[roi_idx].node_number_list] * n_qoi,
+                                                               fname_fsl_gm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
+                                                               fname_fsl_wm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
+                                                               delta=subject.roi[mesh_idx][roi_idx]['delta'],
+                                                               input_data_in_center=True,
+                                                               return_data_in_center=True,
+                                                               data_substitute=-1)
 
     # recreate complete midlayer surface to write in .hdf5 geo file
     points_midlayer, con_midlayer = pynibs.make_GM_WM_surface(
