@@ -1,13 +1,15 @@
-import numpy as np
-import platform
-import scipy
 import os
-import time
 import h5py
-import nibabel as nib
-import trimesh
 import tqdm
+import time
+import scipy
+import trimesh
+import nibabel
+import platform
+import numpy as np
+import nibabel as nib
 from itertools import product
+
 import pynibs
 
 
@@ -43,12 +45,6 @@ def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
 
     ourvertices[idx_label] = True
 
-    # ourvertices = vertices(ourvertices) + 1; % matlab
-    # indexing
-    # 
-    # overlay = zeros(size(vertices));
-    # overlay(ourvertices) = 1;
-
 
 def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_total, N_CPU):
     """
@@ -56,16 +52,16 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
     ----------
     chunk: np.ndarray
         Indices of points the CPU thread is computing the element indices for
-    points_out: np.ndarray of float [N_points x 3]
-        Coordinates of points, the tetrahedra indices are computed for
-    P1_all : np.ndarray of float [N_tet x 3]
-        Coordinates of first point of tetrahedra
-    P2_all : np.ndarray of float [N_tet x 3]
-        Coordinates of second point of tetrahedra
-    P3_all : np.ndarray of float [N_tet x 3]
-        Coordinates of third point of tetrahedra
-    P4_all : np.ndarray of float [N_tet x 3]
-        Coordinates of fourth point of tetrahedra
+    points_out: np.ndarray of float
+     (N_points, 3) Coordinates of points, the tetrahedra indices are computed for
+    P1_all : np.ndarray of float
+    (N_tet, 3) Coordinates of first point of tetrahedra
+    P2_all : np.ndarray of float
+     (N_tet, 3) Coordinates of second point of tetrahedra
+    P3_all : np.ndarray of float
+     (N_tet, 3) Coordinates of third point of tetrahedra
+    P4_all : np.ndarray of float
+     (N_tet, 3) Coordinates of fourth point of tetrahedra
     N_points_total : int
         Total number of points
     N_CPU : int
@@ -73,7 +69,7 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
 
     Returns
     -------
-    tet_idx_local: np.ndarray of int [N_points]
+    tet_idx_local : np.ndarray of int (N_points,)
     """
 
     tet_idx_local = np.zeros([chunk.shape[0]])
@@ -82,31 +78,31 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
     for i in chunk:
         start = time.time()
 
-        vtest1 = pynibs.mesh.calc_tetrahedra_volume_cross(np.tile(points_out[i, :], (P1_all.shape[0], 1)),
-                                                          P2_all,
-                                                          P3_all,
-                                                          P4_all)
+        vtest1 = pynibs.mesh.utils.calc_tetrahedra_volume_cross(np.tile(points_out[i, :], (P1_all.shape[0], 1)),
+                                                                P2_all,
+                                                                P3_all,
+                                                                P4_all)
         tet_idx_bool_1 = (vtest1 >= 0)
         tet_idx_1 = np.nonzero(tet_idx_bool_1)[0]
 
-        vtest2 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_1, :],
-                                                          np.tile(points_out[i, :], (tet_idx_1.shape[0], 1)),
-                                                          P3_all[tet_idx_1, :],
-                                                          P4_all[tet_idx_1, :])
+        vtest2 = pynibs.mesh.utils.calc_tetrahedra_volume_cross(P1_all[tet_idx_1, :],
+                                                                np.tile(points_out[i, :], (tet_idx_1.shape[0], 1)),
+                                                                P3_all[tet_idx_1, :],
+                                                                P4_all[tet_idx_1, :])
         tet_idx_bool_2 = (vtest2 >= 0)
         tet_idx_2 = tet_idx_1[np.nonzero(tet_idx_bool_2)[0]]
 
-        vtest3 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_2, :],
-                                                          P2_all[tet_idx_2, :],
-                                                          np.tile(points_out[i, :], (tet_idx_2.shape[0], 1)),
-                                                          P4_all[tet_idx_2, :])
+        vtest3 = pynibs.mesh.utils.calc_tetrahedra_volume_cross(P1_all[tet_idx_2, :],
+                                                                P2_all[tet_idx_2, :],
+                                                                np.tile(points_out[i, :], (tet_idx_2.shape[0], 1)),
+                                                                P4_all[tet_idx_2, :])
         tet_idx_bool_3 = (vtest3 >= 0)
         tet_idx_3 = tet_idx_2[np.nonzero(tet_idx_bool_3)[0]]
 
-        vtest4 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_3, :],
-                                                          P2_all[tet_idx_3, :],
-                                                          P3_all[tet_idx_3, :],
-                                                          np.tile(points_out[i, :], (tet_idx_3.shape[0], 1)))
+        vtest4 = pynibs.mesh.utils.calc_tetrahedra_volume_cross(P1_all[tet_idx_3, :],
+                                                                P2_all[tet_idx_3, :],
+                                                                P3_all[tet_idx_3, :],
+                                                                np.tile(points_out[i, :], (tet_idx_3.shape[0], 1)))
         tet_idx_bool_4 = (vtest4 >= 0)
 
         tet_idx_local[i_local] = tet_idx_3[
@@ -215,9 +211,10 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
 
     Parameters
     ----------
+    msh : pynibs.TetrahedraLinear
     fname : str or None
         Filename of saved .txt file containing the element indices (no data is saved when fname=None or fname='')
-    points : np.ndarray [N x 3] or list of ndarray
+    points : np.ndarray (N, 3) or list of np.ndarray
         List of points to be queried
     compute_baricentric : bool
         Wether or not to compute baricentric coordinates of the points
@@ -227,7 +224,7 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
     th_with_points : np.ndarray
         List with the tetrahedron that contains each point. If the point is outside
         the mesh, the value will be -1
-    baricentric : np.ndarray [n x 4](if compute_baricentric == True)
+    baricentric : np.ndarray [n, 4](if compute_baricentric == True)
         Baricentric coordinates of point. If the point is outside, a list of zeros
 
     Notes
@@ -242,7 +239,6 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
 
         if int(simnibs.__version__[0]) < 4:
             import simnibs.cython_code.cython_msh as cython_msh
-            # import pynibs.pckg.simnibs.cython_code.cython_msh as cython_msh
         else:
             from simnibs import cython_msh
 
@@ -255,7 +251,7 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
     n_pointsets = len(points)
 
     # determine size of input data
-    n_points = [points[i].shape[0] for i in range(len(points))]
+    n_points = [i.shape[0] for i in points]
     n_points_out_max = np.max(n_points)
 
     tet_idx = np.ones((n_points_out_max, n_pointsets))
@@ -397,7 +393,7 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
         midlayer_surf_fname = [midlayer_surf_fname]
 
     if len(gm_surf_fname) != len(wm_surf_fname):
-        raise Exception('provide equal number of GM and WM surfaces!')
+        raise ValueError('Provide equal number of GM and WM surfaces!')
 
     # load surface data
     points_gm = [None for _ in range(len(gm_surf_fname))]
@@ -413,18 +409,33 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
     for i in range(len(gm_surf_fname)):
         if gm_surf_fname[i] is not None:
-            points_gm[i], con_gm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, gm_surf_fname[i]))
+            if gm_surf_fname[i].endswith('.gii'):
+                img = nibabel.gifti.giftiio.read(os.path.join(mesh_folder, gm_surf_fname[i]))
+                points_gm[i] = img.agg_data('pointset')
+                con_gm[i] = img.agg_data('triangle')
+            else:
+                points_gm[i], con_gm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, gm_surf_fname[i]))
             con_gm[i] = con_gm[i] + max_idx_gm
             max_idx_gm = max_idx_gm + points_gm[i].shape[0]  # np.max(con_gm[i]) + 2
 
         if wm_surf_fname[i] is not None:
-            points_wm[i], con_wm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, wm_surf_fname[i]))
+            if wm_surf_fname[i].endswith('.gii'):
+                img = nibabel.gifti.giftiio.read(os.path.join(mesh_folder, wm_surf_fname[i]))
+                points_wm[i] = img.agg_data('pointset')
+                con_wm[i] = img.agg_data('triangle')
+            else:
+                points_wm[i], con_wm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, wm_surf_fname[i]))
             con_wm[i] = con_wm[i] + max_idx_wm
             max_idx_wm = max_idx_wm + points_wm[i].shape[0]  # np.max(con_wm[i]) + 2
 
         if midlayer_surf_fname[i] is not None:
-            points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(
-                    os.path.join(mesh_folder, midlayer_surf_fname[i]))
+            if midlayer_surf_fname[i].endswith('.gii'):
+                img = nibabel.gifti.giftiio.read(os.path.join(mesh_folder, midlayer_surf_fname[i]))
+                points_mid[i] = img.agg_data('pointset')
+                con_mid[i] = img.agg_data('triangle')
+            else:
+                points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(
+                        os.path.join(mesh_folder, midlayer_surf_fname[i]))
             con_mid[i] = con_mid[i] + max_idx_mid
             max_idx_mid = max_idx_mid + points_mid[i].shape[0]  # np.max(con_wm[i]) + 2
 
@@ -432,7 +443,6 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
     points_wm = np.vstack(points_wm)
     points_mid = np.vstack(points_mid)
     con_gm = np.vstack(con_gm)
-    con_wm = np.vstack(con_wm)
     con_mid = np.vstack(con_mid)
 
     # Determine 3 layer midlayer if GM and WM surfaces are present otherwise use provided midlayer data
@@ -523,17 +533,19 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
         mesh = trimesh.Trimesh(vertices=surface_points_middle,
                                faces=con_cropped_reform)
-        mesh.export(os.path.join(mesh_folder, "roi", "tmp", "roi.stl"))
+        roi_fn = os.path.join(mesh_folder, "roi", "tmp", "roi.stl")
+        mesh.export(roi_fn)
 
-        pynibs.mesh.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
-                                   fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
+        roi_refined_fn = os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl")
+        pynibs.mesh.refine_surface(fn_surf=roi_fn,
+                                   fn_surf_refined=roi_refined_fn,
                                    center=[0, 0, 0],
                                    radius=np.inf,
                                    verbose=True,
                                    repair=False,
                                    remesh=False)
 
-        roi = trimesh.load(os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"))
+        roi = trimesh.load(roi_refined_fn)
         con_cropped_reform = roi.faces
         surface_points_middle = roi.vertices
 
@@ -543,16 +555,16 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
             for p in [surface_points_upper, surface_points_lower]:
                 mesh = trimesh.Trimesh(vertices=p,
                                        faces=con_cropped_reform)
-                mesh.export(os.path.join(mesh_folder, "roi", "tmp", "roi.stl"))
+                mesh.export(roi_fn)
 
-                pynibs.mesh.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
-                                           fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
+                pynibs.mesh.refine_surface(fn_surf=roi_fn,
+                                           fn_surf_refined=roi_refined_fn,
                                            center=[0, 0, 0],
                                            radius=np.inf,
                                            verbose=True,
                                            repair=False)
 
-                roi.append(trimesh.load(os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl")))
+                roi.append(trimesh.load(roi_refined_fn))
 
             surface_points_upper = roi[0].vertices
             surface_points_lower = roi[1].vertices
@@ -569,14 +581,14 @@ class RegionOfInterestSurface:
 
     Attributes
     ----------
-    node_coord_up : np.ndarray [N_points x 3]
-        Coordinates (x,y,z) of upper surface nodes
-    node_coord_mid : np.ndarray [N_points x 3]
-        Coordinates (x,y,z) of middle surface nodes
-    node_coord_low : np.ndarray [N_points x 3]
-        Coordinates (x,y,z) of lower surface nodes
-    node_number_list : np.ndarray [N_points x 3]
-        Connectivity matrix of triangles
+    node_coord_up : np.ndarray
+     (N_points, 3) Coordinates (x,y,z) of upper surface nodes
+    node_coord_mid : np.ndarray
+     (N_points, 3) Coordinates (x,y,z) of middle surface nodes
+    node_coord_low : np.ndarray
+     (N_points, 3) Coordinates (x,y,z) of lower surface nodes
+    node_number_list : np.ndarray
+     (N_points, 3) Connectivity matrix of triangles
     delta : float
         Distance parameter between WM and GM (0 -> WM, 1 -> GM)
     tet_idx_tri_center_up : np.ndarray [N_points]
@@ -588,14 +600,14 @@ class RegionOfInterestSurface:
     tet_idx_tri_center_low : np.ndarray [N_points]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
         lower surface are lying in
-    tet_idx_node_coord_mid : np.ndarray [N_tri]
-        Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle surface are lying in
-    tri_center_coord_up : np.ndarray [N_tri x 3]
-        Coordinates of roi triangle center of upper surface
-    tri_center_coord_mid : np.ndarray [N_tri x 3]
-        Coordinates of roi triangle center of middle surface
-    tri_center_coord_low : np.ndarray [N_tri x 3]
-        Coordinates of roi triangle center of lower surface
+    tet_idx_node_coord_mid : np.ndarray
+     (N_tri,) Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle surface are lying in
+    tri_center_coord_up : np.ndarray
+     (N_tri, 3) Coordinates of roi triangle center of upper surface
+    tri_center_coord_mid : np.ndarray
+     (N_tri, 3) Coordinates of roi triangle center of middle surface
+    tri_center_coord_low : np.ndarray
+     (N_tri, 3) Coordinates of roi triangle center of lower surface
     fn_mask : string
         Filename for surface mask in subject space. .mgh file or freesurfer surface file.
     fn_mask_avg : string
@@ -638,18 +650,18 @@ class RegionOfInterestSurface:
     def __init__(self):
         """ Initialize RegionOfInterestSurface class instance """
 
-        self.node_coord_up = []
-        self.node_coord_mid = []
-        self.node_coord_low = []
+        self.node_coord_up = np.empty(0)
+        self.node_coord_mid = np.empty(0)
+        self.node_coord_low = np.empty(0)
 
-        self.node_number_list = []
+        self.node_number_list = np.empty(0)
         self.delta = []
 
-        self.tet_idx_tri_center_up = []
-        self.tet_idx_tri_center_mid = []
-        self.tet_idx_tri_center_low = []
+        self.tet_idx_tri_center_up = np.empty(0)
+        self.tet_idx_tri_center_mid = np.empty(0)
+        self.tet_idx_tri_center_low = np.empty(0)
 
-        self.tet_idx_node_coord_mid = []
+        self.tet_idx_node_coord_mid = np.empty(0)
 
         self.tri_center_coord_up = []
         self.tri_center_coord_mid = []
@@ -813,18 +825,16 @@ class RegionOfInterestSurface:
             midlayer_surf_fname = [midlayer_surf_fname]
 
         if len(gm_surf_fname) != len(wm_surf_fname):
-            raise Exception('provide equal number of GM and WM surfaces!')
+            raise ValueError('provide equal number of GM and WM surfaces!')
 
         # load surface data
         points_gm = [None for _ in range(len(gm_surf_fname))]
         points_wm = [None for _ in range(len(wm_surf_fname))]
         points_mid = [None for _ in range(len(midlayer_surf_fname))]
         con_gm = [None for _ in range(len(gm_surf_fname))]
-        # con_wm = [None for _ in range(len(wm_surf_fname))]
         con_mid = [None for _ in range(len(midlayer_surf_fname))]
 
         max_idx_gm = 0
-        # max_idx_wm = 0
         max_idx_mid = 0
 
         def read_geom(fn, max_idx):
@@ -860,7 +870,6 @@ class RegionOfInterestSurface:
         points_wm = np.vstack(points_wm)
         points_mid = np.vstack(points_mid)
         con_gm = np.vstack(con_gm)
-        # con_wm = np.vstack(con_wm) # we don't use this
         con_mid = np.vstack(con_mid)
 
         # Determine 3 layer midlayer if GM and WM surfaces are present otherwise use provided midlayer data
@@ -959,16 +968,18 @@ class RegionOfInterestSurface:
 
             mesh = trimesh.Trimesh(vertices=surface_points_middle,
                                    faces=con_cropped_reform)
-            mesh.export(os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"))
+            roi_fn = os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl")
+            mesh.export(roi_fn)
 
-            pynibs.mesh.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
-                                       fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"),
+            roi_refined_fn = os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl")
+            pynibs.mesh.refine_surface(fn_surf=roi_fn,
+                                       fn_surf_refined=roi_refined_fn,
                                        center=[0, 0, 0],
                                        radius=np.inf,
                                        verbose=True,
                                        repair=False)
 
-            roi = trimesh.load(os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"))
+            roi = trimesh.load(roi_refined_fn)
             con_cropped_reform = roi.faces
             surface_points_middle = roi.vertices
 
@@ -978,17 +989,16 @@ class RegionOfInterestSurface:
                 for p in [surface_points_upper, surface_points_lower]:
                     mesh = trimesh.Trimesh(vertices=p,
                                            faces=con_cropped_reform)
-                    mesh.export(os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"))
+                    mesh.export(roi_fn)
 
-                    pynibs.mesh.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
-                                               fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp",
-                                                                            "roi_refined.stl"),
+                    pynibs.mesh.refine_surface(fn_surf=roi_fn,
+                                               fn_surf_refined=roi_refined_fn,
                                                center=[0, 0, 0],
                                                radius=np.inf,
                                                verbose=True,
                                                repair=False)
 
-                    roi.append(trimesh.load(os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl")))
+                    roi.append(trimesh.load(roi_refined_fn))
 
                 surface_points_upper = roi[0].vertices
                 surface_points_lower = roi[1].vertices
@@ -1085,8 +1095,8 @@ class RegionOfInterestSurface:
         r = np.linalg.norm(sphere_coords[0, :])
         a0 = 4 * np.pi * r ** 2
 
-        sphere_points_sampled = pynibs.sample_sphere(n_points=int(np.ceil(a0 / dist ** 2) // 2 * 2 + 1),
-                                                     r=np.linalg.norm(sphere_coords[0, :]))
+        sphere_points_sampled = pynibs.mesh.utils.sample_sphere(n_points=int(np.ceil(a0 / dist ** 2) // 2 * 2 + 1),
+                                                                r=np.linalg.norm(sphere_coords[0, :]))
 
         # read mask from freesurfer mask file
         mask = nib.freesurfer.mghformat.MGHImage.from_filename(
@@ -1227,134 +1237,12 @@ class RegionOfInterestVolume:
         self.tet_idx_tetrahedra_center = np.array(tet_con_row_idx)
         self.tet_idx_triangle_center = np.array(tri_con_row_idx)
 
-    # def make_roi_volume_from_points(self, msh, points):
-    #     """
-    #     Generate region of interest (volume) and extract nodes, triangles and tetrahedra from msh instance
-    #
-    #     Parameters
-    #     ----------
-    #     msh: object instance
-    #         Mesh object instance of type TetrahedraLinear (see main.py)
-    #     points: np.ndarray [N_points_roi x 3]
-    #         Coordinates of points in ROI x, y, z)
-    #     """
-    #     a=1
-    #
-    # def determine_element_idx_in_mesh(self):
-    #     """ In development """
-    #     a=1
-
-
-# def determine_element_idx_in_mesh_old(self, fname, points, N_MAX_CPU=20):
-#     """
-#     Determine the element indices where the points_out are lying and safe the results to a .txt file.
-#
-#     Parameters
-#     ----------
-#     fname: str
-#         Filename of saved .txt file containing the element indices
-#     points_out: array(s) of float (list possible)
-#         Point coordinates (x,y,z) whose corresponding elements are found
-#     N_MAX_CPU: int
-#         Number of CPU cores to determine the element indices
-#
-#     Returns
-#     -------
-#     <File>: .txt file
-#         Element indices of the mesh, where the points are lying in
-#     tet_idx: ndarray [N_points x N_point_tupels]
-#         Element indices of the mesh, where the points are lying in
-#     """
-#
-#     if type(points) is not list:
-#         points = [points]
-#
-#     # determine size of input data
-#     N_points = [points[i].shape[0] for i in range(len(points))]
-#     N_points_out_max = np.max(N_points)
-#
-#     # initialize tet_idx array
-#     tet_idx = np.ones([N_points_out_max, len(points)]) * -1
-#
-#     P1_all = self.points[self.tetrahedra[:, 0], :]
-#     P2_all = self.points[self.tetrahedra[:, 1], :]
-#     P3_all = self.points[self.tetrahedra[:, 2], :]
-#     P4_all = self.points[self.tetrahedra[:, 3], :]
-#
-#     # determine all volumes (replacing points with points_out)
-#     # find the element where all volumes are > 0 (not inverted element)
-#     # get index of this tetrahedron
-#     # do it successively to decrease amount of volume calculations for all
-#     # 4 points in tetrahedra
-#
-#     N_CPU_total = multiprocessing.cpu_count()
-#     if N_CPU_total > N_MAX_CPU:
-#         N_CPU = N_MAX_CPU
-#     else:
-#         N_CPU = N_CPU_total
-#
-#     if platform.system() == 'Windows':
-#         print("Starting single core version with {:d}/{:d} processors ...").format(1, N_CPU_total)
-#
-#         for j in range(len(points)):
-#             chunks = np.arange(len(points[j]))
-#             tet_idx[:, j] = elem_workhorse(chunk=chunks,
-#                                            points_out=points[j],
-#                                            P1_all=P1_all,
-#                                            P2_all=P2_all,
-#                                            P3_all=P3_all,
-#                                            P4_all=P4_all,
-#                                            N_points_total=N_points[j],
-#                                            N_CPU=N_CPU)
-#
-#     else:
-#         print("Starting multiprocessed index version with {:d}/{:d} processors ...").format(N_CPU, N_CPU_total)
-#
-#         for j in range(len(points)):
-#             print("Starting run " + str(j + 1) + " of " + str(range(len(points))))
-#             # n_points[j] are all points in roi
-#
-#             # range(N_points_out_max) aufteilen nach cpu count
-#             avg = N_points[j] / float(N_CPU)
-#             chunks = []
-#             last = 0.0
-#             points_idx = range(N_points[j])
-#             while last < N_points[j]:
-#                 chunks.append(np.array(points_idx[int(last):int(last + avg)]))
-#                 last += avg
-#
-#             # call pool
-#             pool = multiprocessing.Pool(N_CPU)
-#             partialized = partial(elem_workhorse,
-#                                   points_out=points[j],
-#                                   P1_all=P1_all,
-#                                   P2_all=P2_all,
-#                                   P3_all=P3_all,
-#                                   P4_all=P4_all,
-#                                   N_points_total=N_points[j],
-#                                   N_CPU=N_CPU)
-#
-#             results = pool.map(partialized, chunks)
-#             pool.close()
-#             pool.join()
-#
-#             # concatenate results
-#             tet_idx[:, j] = np.concatenate(results, axis=0)
-#
-#     # save results
-#     if not os.path.exists(os.path.dirname(fname)):
-#         os.makedirs(os.path.dirname(fname))
-#     np.savetxt(fname, tet_idx, '%d')
-#     print 'Saved element indices of points_out in {}'.format(fname)
-#
-#     return tet_idx
-
 
 def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
                       thresh_by_nii=True, val_in=1, val_out=0,
                       outside_val=0, outside_radius=np.inf):
     """
-    Computes a spherical ROI for a given Nifti image (defaults to simnibs MNI T1 tissue). The ROI area is defined
+    Computes a spherical ROI for a given Nifti image (defaults to SimNIBS MNI T1 tissue). The ROI area is defined
     in nifti coordinates.
     By default, everything inside the ROI is set to 1, areas outside = 0.
     The ROI is further thresholded by the nifti.
@@ -1394,11 +1282,11 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
     ValueError
         If the final ROI is empty.
     """
-    from simnibs import SIMNIBSDIR
     if outside_radius is None:
         outside_radius = np.inf
     # load image for affine and tissue borders
     if nii is None:
+        from simnibs import SIMNIBSDIR
         mni_fn = f"{SIMNIBSDIR}/resources/templates/spmprior_tissue.nii"
         nii = nib.load(mni_fn)
     elif isinstance(nii, str):
@@ -1425,15 +1313,15 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
 
     else:
         # if outside_val should not be set we only need to search within center-radius
-        x_min, y_min, z_min = center_vox - radius
-        x_max, y_max, z_max = center_vox + radius
+        x_min, y_min, z_min = (center_vox - radius).astype(int)
+        x_max, y_max, z_max = (center_vox + radius).astype(int)
 
     # compute spherical roi around nifti coordinates
     # do this in original coordinates to have correct mm radius
 
-    pixdim = nii.get_header()['pixdim'][1:4]
+    pixdim = nii.header['pixdim'][1:4]
     for xyz in tqdm.tqdm(product(range(x_min, x_max), range(y_min, y_max), range(z_min, z_max)),
-                         total=x_max * y_max * z_max,
+                         total=(x_max-x_min) * (y_max-y_min) * (z_max - z_min),
                          desc="Finding ROI elements."):
         # set to val_in if within radius
         dist = norm(np.multiply(pixdim, center_vox - xyz))
@@ -1611,6 +1499,6 @@ def nii2msh(mesh, m2m_dir, nii, out_folder, hem, out_fsaverage=False, roi_name='
     else:
         out_fsaverage = None
     transformations.middle_gm_interpolation(fn_mesh_out, m2m_folder=m2m_dir,
-                                            out_folder=out_folder, out_fsaverage=out_folder)
+                                            out_folder=out_folder, out_fsaverage=out_fsaverage)
 
     return nib.freesurfer.read_morph_data(os.path.join(out_folder, f"{hem}.mesh.central.{roi_name}"))
