@@ -276,27 +276,32 @@ class Subject:
             self.exp[exp] = exp_dict[exp]
 
 
-def fill_from_dict(self, d):
+def fill_from_dict(obj, d):
     """
-    Set all attributes from d in object self.
+    Set all attributes from d in obj.
 
-    Parameters:
-    -----------
+    Parameters
+    ----------
+    obj : pynibs.Mesh or pynibs.ROI
     d : dict
+
+    Returns
+    -------
+    obj : pynibs.Mesh or pynibs.ROI
     """
 
     for key, value in d.items():
-        if not hasattr(self, f"{key}"):
+        if not hasattr(obj, f"{key}"):
             warnings.warn(f"{key} not existing.")
-        setattr(self, key, value)
+        setattr(obj, key, value)
 
-    return self
+    return obj
 
 
 def save_subject(subject_id, subject_folder, fname, mri_dict=None, mesh_dict=None, roi_dict=None,
                  exp_dict=None, ps_dict=None, **kwargs):
     """
-    Saving subject information in .pkl or .hdf5 format (preferred)
+    Saves subject information in .pkl or .hdf5 format (preferred)
 
     Parameters
     ----------
